local RW = LibStub("AceAddon-3.0"):GetAddon("Raid Watch")
local L = LibStub("AceLocale-3.0"):GetLocale("RW2-Plugin-Base")

local Boss = RW.BossPrototype
local iconProt = {}
local mt = {__index = iconProt}

local Party = RW:GetPlugin("Party")
local Utils = RW.Utils

local IconToNum  = {
	STAR	= 1,
	CIRCLE 	= 2,
	DIAMOND	= 3,
	TRIANGLE = 4,
	MOON 	= 5,
	SQUARE 	= 6,
	CROSS 	= 7,
	SKULL 	= 8,
}

local iconCoords = {
	STAR		= {0, 		0.25,	0, 		0.25},
	CIRCLE 		= {0.25,	0.5,	0,		0.25},
	DIAMOND		= {0.5, 	0.75,	0,		0.25},
	TRIANGLE 	= {0.75,	1,		0,		0.25},
	MOON 		= {0,		0.25,	0.25,	0.5},
	SQUARE 		= {0.25,	0.5,	0.25,	0.5},
	CROSS 		= {0.5,		0.75,	0.25,	0.5},
	SKULL 		= {0.75,	1,		0.25,	0.5},
}

function Boss:_Icon(defaultIcon, optionName, timeOut, ...)	
	local icons = {}
	icons[defaultIcon] = true
	for i = 1, select("#", ...) do
		local ic = select(i, ...)
		icons[ic] = true
	end
	
	local base = self:GetBaseAlertObject(optionName)
	if not base then
		base = {		
			id = optionName,
			mod = self,
			scheduled = {},
		}
		tinsert(self.BaseAlertObjects, base)
	end
	
	obj = setmetatable({
		mainType = "ICON",
		type = defaultIcon,
		icons = icons,
		usedIcons = {},
		timeOut = timeOut,
		oldIcons = {},
		base = base, 
	}, mt)
	
	self.AlertObjects.ICON = self.AlertObjects.ICON or {}
	tinsert(self.AlertObjects.ICON, obj)
	
	return obj
end

function Boss:Icon(defaultIcon, optionName, default, timeOut, ...)
	
	for i = 1, select("#", ...) do
		local n = select(i, ...)
		if type(n) == "string" and IconToNum[n] then
			self.usedIcons = self.usedIcons or {}
			self.usedIcons[n] = true
		end
	end
	
	local spell
	local icon
	if type(optionName) == "number" then
		spell = optionName
		icon = select(3, GetSpellInfo(optionName))
		optionName = Utils.SpellName[optionName]
	end
	
	-- If no option name is found by now we use the text as default
	if not optionName then optionName = type(text) == "table" and text[1] or text end
	
	self:AddToCategory(nil, optionName)
	
	local defaults = self:AddDefaults(optionName, "ICON", defaultIcon, default)
	
	self.usedIcons = self.usedIcons or {}
	self.usedIcons[defaultIcon] = true
	
	local obj = self:_Icon(defaultIcon, optionName, timeOut, ...)
	
	obj.base.icon = icon
	obj.base.spell = obj.base.spell or spell
	
	return obj
end

-- Reset all icons, both reset the cache and put old marks back on people
function Boss:ResetAllIcons()
	if self.icons then
		for i, icon in pairs(self.icons) do
			for target, oldIcon in pairs(icon.oldIcons) do
				SetRaidTarget(target, oldIcon)
			end
			wipe(icon.oldIcons)
			resetIcons(icon)
		end
	end
end

-- Puts the old mark back on people or just hides it if they did not have one when they got marked
-- Old mark will not be put back if it is used by someone else at that point
local function hideIcon(self, target)
	local used = false
	if not used then
		SetRaidTarget(target, self.oldIcons[target] or 0)
	else
		SetRaidTarget(target, 0)
	end
	self.oldIcons[target] = nil
end

-- Reset the icon cache
function resetIcons(self)
	for icon in pairs(self.usedIcons) do
		self.icons[icon] = true
	end
	wipe(self.usedIcons)
end

function iconProt:Show(target, time, icon, ...)
	if not self.base.mod.DB[self.base.id].enabled then return end
	if not self.base.mod.DB[self.base.id].ICON[self.optText or self.type].enabled then return end
	if self.base.mod.Party.group[self.base.mod.pName] and self.base.mod.Party.group[self.base.mod.pName].rank < 1 and self.base.mod.Party:GetPartyType() == "raid" then
		return
	end
		
	local targetID = Boss:UIDFromRandom(target)
	
	if targetID and Utils:NrElements(self.icons) > 0 then
		self.oldIcons[targetID] = GetRaidTargetIndex(targetID) or 0
		
		local icn
		if icon then
			if self.icons[icon] then
				icn = icon
				self.usedIcons[icn] = true
				self.icons[icn] = nil
			end
		else
			icn = next(self.icons)
			self.icons[icn] = nil
			self.usedIcons[icn] = true
		end
		if icn then
			SetRaidTarget(targetID, IconToNum[icn] or 0)
			if time or self.timeOut then
				self.base.mod:Schedule("ICON"..self.base.id, time or self.timeOut, hideIcon, self, targetID)
			end
			self.base.mod:Schedule(self.base.id.."ResetIcons", 2, resetIcons, self)
		end
	end
end
iconProt.Start = iconProt.Show

function iconProt:Hide(extraID)
	if not extraID then
		for id, icon in pairs(self.oldIcons) do
			SetRaidTarget(id, icon)
		end
		wipe(self.oldIcons)
		return
	end

	local targetID = Boss:UIDFromRandom(extraID)
	
	for id, icon in pairs(self.oldIcons) do
		if targetID == id then
			SetRaidTarget(id, icon)
			self.oldIcons[id] = nil
			break
		end
	end
end

function iconProt:Schedule(stime, info, ...)
	self.base.mod:Schedule("ICON"..self.base.id..(info or ""), stime, showMessage, self, info, ...)
	tinsert(self.base.scheduled, "ICON"..self.base.id..(info or ""))
end

function iconProt:ScheduleRepeating(stime, now, info, ...)
	self.base.mod:ScheduleRepeating("ICON"..self.base.id..(info or ""), stime, showMessage, now, self, info, ...)
	tinsert(self.base.scheduled, "ICON"..self.base.id..(info or ""))
end

function iconProt:Unschedule(extraID)
	self.base.mod:Unschedule("ICON"..self.base.id..(extraID or ""))
	Utils:TRemove(self.base.scheduled, "ICON"..self.base.id..(extraID or ""))
end

-- Options
function iconProt:GetOption()
	local Options = RW:GetPlugin("Options")
	local boss = self.base.mod
	local AceGUI = LibStub("AceGUI-3.0")
	local UIF = LibStub("AceGUI-Factory-1.0"):GetFactory("RW")

	local subGroup = UIF:InlineGroup("")
	
	local check = UIF:CheckBox(L.ENABLED, boss.DB[self.base.id].ICON[self.optText or self.type], "enabled")
	
	
	local sg = AceGUI:Create("SimpleGroup")	
	sg:SetFullWidth(true)
	sg:SetLayout("Flow")
	for icn in pairs(self.icons) do
		local iconWidget = AceGUI:Create("Icon")
		iconWidget:SetImage("Interface\\TargetingFrame\\UI-RaidTargetingIcons.blp", unpack(iconCoords[icn]))
		iconWidget:SetImageSize(45, 45)
		iconWidget:SetHeight(50)
		iconWidget:SetRelativeWidth(0.24)
		iconWidget:SetText("")
		sg:AddChild(iconWidget)
	end
		
	subGroup:AddChild(check)
	subGroup:AddChild(sg)
	
	return subGroup
end 